@@ -22,21 +22,45 @@
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
+$string['acceptandaddcontact'] = 'Accept and add to contacts';
 $string['addcontact'] = 'Add contact';
-$string['addtoyourcontacts'] = 'Add to your contacts';
+$string['addcontactconfirm'] = 'Are you sure you want to add {$a} to your contacts?';
+$string['addtoyourcontacts'] = 'Add to contacts';
+$string['addtoyourcontactsandmessage'] = 'Add to contacts and message';
+$string['addtofavourites'] = 'Star conversation';
 $string['ago'] = '{$a} ago';
 $string['allusers'] = 'All messages from all users';
+$string['backto'] = 'Back to {$a}';
 $string['backupmessageshelp'] = 'If enabled, then instant messages will be included in SITE automated backups';
 $string['blockcontact'] = 'Block contact';
+$string['blockedusers'] = 'Blocked users';
+$string['blockuser'] = 'Block user';
+$string['blockuserconfirm'] = 'Are you sure you want to block {$a}?';
+$string['blockuserconfirmbutton'] = 'Block';
 $string['blocknoncontacts'] = 'Prevent non-contacts from messaging me';
-$string['canceledit'] = 'Cancel editing messages';
+$string['cancelselection'] = 'Cancel message selection';
+$string['cantblockuser'] = 'You can\'t block {$a} because they have a role with permission to message all users.';
+$string['contactableprivacy'] = 'Accept messages from:';
+$string['contactableprivacy_onlycontacts'] = 'My contacts only';
+$string['contactableprivacy_coursemember'] = 'My contacts and anyone in my courses';
+$string['contactableprivacy_site'] = 'Anyone on the site';
 $string['contactblocked'] = 'Contact blocked';
+$string['contactrequests'] = 'Contact requests';
+$string['contactrequestsent'] = 'Contact request sent';
 $string['contacts'] = 'Contacts';
-$string['defaultmessageoutputs'] = 'Default message outputs';
+$string['conversationactions'] = 'Conversation actions menu';
+$string['decline'] = 'Decline';
+$string['defaultmessageoutputs'] = 'Notification settings';
 $string['defaults'] = 'Defaults';
-$string['deleteallconfirm'] = "Are you sure you would like to delete this entire conversation?";
+$string['deleteallconfirm'] = "Are you sure you would like to delete this entire conversation? This will not delete it for other conversation participants.";
 $string['deleteallmessages'] = "Delete all messages";
+$string['deleteallselfconfirm'] = "Are you sure you would like to delete this entire personal conversation?";
+$string['deleteconversation'] = "Delete conversation";
+$string['deleteforeveryone'] = 'Delete for me and for everyone else';
+$string['deleteforeveryoneselectedmessagesconfirm'] = 'Are you sure you would like to delete the selected messages?';
 $string['deleteselectedmessages'] = 'Delete selected messages';
+$string['deleteselectedmessagesconfirm'] = 'Are you sure you would like to delete the selected messages? This will not delete them for other conversation participants.';
+$string['deleteselectedmessagesconfirmselfconversation'] = 'Are you sure you would like to delete the selected personal messages?';
 $string['disableall'] = 'Disable notifications';
 $string['disabled'] = 'Messaging is disabled on this site';
 $string['disallowed'] = 'Disallowed';
@@ -44,61 +68,95 @@ $string['editmessages'] = 'Edit messages';
 $string['emailtagline'] = 'This is a copy of a message sent to you at "{$a->sitename}". Go to {$a->url} to reply.';
 $string['enabled'] = 'Enabled';
 $string['errorcallingprocessor'] = 'Error calling defined output';
+$string['errorconversationdoesnotexist'] = 'Conversation does not exist';
+$string['errormessagetoolong'] = 'The message is longer than the maximum allowed.';
 $string['errortranslatingdefault'] = 'Error translating default setting provided by plugin, using system defaults instead.';
+$string['eventgroupmessagesent'] = 'Group message sent';
 $string['eventnotificationviewed'] = 'Notification viewed';
 $string['eventnotificationsent'] = 'Notification sent';
 $string['eventmessagecontactadded'] = 'Message contact added';
-$string['eventmessagecontactblocked'] = 'Message contact blocked';
 $string['eventmessagecontactremoved'] = 'Message contact removed';
-$string['eventmessagecontactunblocked'] = 'Message contact unblocked';
 $string['eventmessagedeleted'] = 'Message deleted';
+$string['eventmessageuserblocked'] = 'User blocked';
+$string['eventmessageuserunblocked'] = 'User unblocked';
 $string['eventmessageviewed'] = 'Message viewed';
 $string['eventmessagesent'] = 'Message sent';
 $string['forced'] = 'Locked';
 $string['guestnoeditmessage'] = 'Guest user can not edit messaging options';
 $string['guestnoeditmessageother'] = 'Guest user can not edit other user messaging options';
+$string['groupinfo'] = 'Group info';
+$string['groupconversations'] = 'Group';
 $string['hidemessagewindow'] = 'Hide message window';
 $string['hidenotificationwindow'] = 'Hide notification window';
+$string['individualconversations'] = 'Private';
+$string['info'] = 'User info';
+$string['isnotinyourcontacts'] = '{$a} is not in your contacts';
+$string['loadmore'] = 'Load more';
 $string['loggedin'] = 'Online';
 $string['loggedin_help'] = 'Configure how you would like to receive notifications when you are logged into Moodle';
 $string['loggedindescription'] = 'When you are logged into Moodle';
 $string['loggedoff'] = 'Offline';
 $string['loggedoff_help'] = 'Configure how you would like to receive notifications when you are not logged into Moodle';
 $string['loggedoffdescription'] = 'When you are not logged into Moodle';
-$string['managemessageoutputs'] = 'Manage message outputs';
-$string['messageoutputs'] = 'Message outputs';
+$string['managemessageoutputs'] = 'Default notification preferences';
+$string['messageoutputs'] = 'Notification plugins';
 $string['messagepreferences'] = 'Message preferences';
 $string['message'] = 'Message';
+$string['messagecontactrequest'] = '{$a->user} is requesting to be added as a contact. Visit your <a href="{$a->url}">contact requests</a> page to respond to the request';
+$string['messagecontactrequestsubject'] = '{$a->sitename}: Contact request from {$a->user}';
+$string['messagecontentaudio'] = 'Audio';
+$string['messagecontentimage'] = 'Image';
+$string['messagecontentmultimediageneral'] = 'Other media';
+$string['messagecontentvideo'] = 'Video';
+$string['messagedrawerviewcontact'] = 'User details for {$a}';
+$string['messagedrawerviewcontacts'] = 'Message contacts';
+$string['messagedrawerviewconversation'] = 'Conversation with {$a}';
+$string['messagedrawerviewgroupinfo'] = 'Group details for {$a}';
+$string['messagedrawerviewoverview'] = 'Messages overview';
+$string['messagedrawerviewsearch'] = 'Messages search results for {$a}';
+$string['messagedrawerviewsettings'] = 'Message settings';
 $string['messagepreferences'] = 'Message preferences';
 $string['messages'] = 'Messages';
+$string['messagesselected:'] = 'Messages selected:';
+$string['messagetextrequired'] = 'Please enter message text';
 $string['messagingdatahasnotbeenmigrated'] = 'Your messages are temporarily unavailable due to upgrades in the messaging infrastructure. Please wait for them to be migrated.';
-$string['messagingdisabled'] = 'Messaging is disabled on this site, emails will be sent instead';
+$string['muteconversation'] = 'Mute';
+$string['mutedconversation'] = 'Muted conversation';
 $string['newonlymsg'] = 'Show only new';
 $string['newmessage'] = 'New message';
 $string['newmessagesearch'] = 'Select or search for a contact to send a new message.';
-$string['newsearch'] = 'New search';
+$string['nofavourites'] = 'No starred conversations';
 $string['noframesjs'] = 'Use more accessible interface';
+$string['nocontactrequests'] = 'No contact requests';
 $string['nocontacts'] = 'No contacts';
-$string['nomessages'] = 'No messages';
+$string['nocontactsgetstarted'] = 'No contacts';
+$string['nogroupconversations'] = 'No group conversations';
+$string['noindividualconversations'] = 'No private conversations';
 $string['nomessagesfound'] = 'No messages were found';
 $string['noreply'] = 'Do not reply to this message';
 $string['noncontacts'] = 'Non-contacts';
 $string['nonotifications'] = 'You have no notifications';
+$string['noparticipants'] = 'No participants';
 $string['notificationdatahasnotbeenmigrated'] = 'Your notifications are temporarily unavailable due to upgrades in the notification infrastructure. Please wait for them to be migrated.';
 $string['notificationwindow'] = 'Notification window';
 $string['notificationpreferences'] = 'Notification preferences';
 $string['notificationimage'] = 'Notification image';
 $string['notifications'] = 'Notifications';
+$string['notincontactsheading'] = '{$a} is not in your contacts';
+$string['notincontacts'] = 'You need to add {$a} to your contacts to be able to send them messages.';
+$string['numparticipants'] = '{$a} participants';
 $string['off'] = 'Off';
 $string['offline'] = 'Offline';
 $string['on'] = 'On';
 $string['online'] = 'Online';
-$string['outputdisabled'] = 'Output disabled';
-$string['outputdoesnotexist'] = 'Message output does not exists';
-$string['outputenabled'] = 'Output enabled';
+$string['otherparticipants'] = 'Other participants';
 $string['outputnotavailable'] = 'Not available';
-$string['outputnotconfigured'] = 'Not configured';
+$string['participants'] = 'Participants';
+$string['pendingcontactrequests'] = 'There are {$a} pending contact requests';
 $string['permitted'] = 'Permitted';
+$string['privacy'] = 'Privacy';
+$string['privacy_desc'] = 'You can restrict who can message you';
+$string['privacy:metadata:core_favourites'] = 'The conversations starred by the user';
 $string['privacy:metadata:messages'] = 'Messages';
 $string['privacy:metadata:messages:conversationid'] = 'The ID of the conversation';
 $string['privacy:metadata:messages:fullmessage'] = 'The full message';
@@ -107,20 +165,34 @@ $string['privacy:metadata:messages:fullmessagehtml'] = 'The HTML format of the f
 $string['privacy:metadata:messages:useridfrom'] = 'The ID of the user who sent the message';
 $string['privacy:metadata:messages:smallmessage'] = 'A small version of the message';
 $string['privacy:metadata:messages:subject'] = 'The subject of the message';
-$string['privacy:metadata:messages:timecreated'] = 'The date at which the message was created';
+$string['privacy:metadata:messages:timecreated'] = 'The time when the message was created';
+$string['privacy:metadata:messages:customdata'] = 'Custom data, usually containing internal IDs and a public URL of the sender image (user or group)';
 $string['privacy:metadata:message_contacts'] = 'The list of contacts';
-$string['privacy:metadata:message_contacts:blocked'] = 'Flag whether or not the user is blocked';
 $string['privacy:metadata:message_contacts:contactid'] = 'The ID of the user who is a contact';
+$string['privacy:metadata:message_contacts:timecreated'] = 'The time when the contact was created';
 $string['privacy:metadata:message_contacts:userid'] = 'The ID of the user whose contact list we are viewing';
+$string['privacy:metadata:message_contact_requests'] = 'The list of contact requests';
+$string['privacy:metadata:message_contact_requests:requesteduserid'] = 'The ID of the user who received the contact request';
+$string['privacy:metadata:message_contact_requests:timecreated'] = 'The time when the contact request was created';
+$string['privacy:metadata:message_contact_requests:userid'] = 'The ID of the user who sent the contact request';
+$string['privacy:metadata:message_conversation_actions'] = 'The list of conversation user actions';
+$string['privacy:metadata:message_conversation_actions:action'] = 'The action that was performed';
+$string['privacy:metadata:message_conversation_actions:conversationid'] = 'The ID of the conversation this action belongs to';
+$string['privacy:metadata:message_conversation_actions:timecreated'] = 'The time when the action was created';
+$string['privacy:metadata:message_conversation_actions:userid'] = 'The ID of the user who performed this action';
 $string['privacy:metadata:message_conversation_members'] = 'The list of users in a conversation';
 $string['privacy:metadata:message_conversation_members:conversationid'] = 'The ID of the conversation';
-$string['privacy:metadata:message_conversation_members:timecreated'] = 'The date at which the member was created';
+$string['privacy:metadata:message_conversation_members:timecreated'] = 'The time when the member was created';
 $string['privacy:metadata:message_conversation_members:userid'] = 'The ID of the user in a conversation';
 $string['privacy:metadata:message_user_actions'] = 'The list of message user actions';
 $string['privacy:metadata:message_user_actions:action'] = 'The action that was performed';
 $string['privacy:metadata:message_user_actions:messageid'] = 'The ID of the message this action belongs to';
-$string['privacy:metadata:message_user_actions:timecreated'] = 'The date at which the action was created';
+$string['privacy:metadata:message_user_actions:timecreated'] = 'The time when the action was created';
 $string['privacy:metadata:message_user_actions:userid'] = 'The ID of the user who performed this action';
+$string['privacy:metadata:message_users_blocked'] = 'The list of blocked users';
+$string['privacy:metadata:message_users_blocked:blockeduserid'] = 'The ID of the user who was blocked';
+$string['privacy:metadata:message_users_blocked:timecreated'] = 'The time when the blocked action was created';
+$string['privacy:metadata:message_users_blocked:userid'] = 'The ID of the user who performed the blocked action';
 $string['privacy:metadata:notifications'] = 'Notifications';
 $string['privacy:metadata:notifications:component'] = 'The component responsible for sending the notification';
 $string['privacy:metadata:notifications:contexturl'] = 'The URL related to this notification';
@@ -131,16 +203,21 @@ $string['privacy:metadata:notifications:fullmessageformat'] = 'The notification
 $string['privacy:metadata:notifications:fullmessagehtml'] = 'The HTML of the notification';
 $string['privacy:metadata:notifications:smallmessage'] = 'The small message of the notification';
 $string['privacy:metadata:notifications:subject'] = 'The subject of the notification';
-$string['privacy:metadata:notifications:timeread'] = 'The date at which the notification was read';
-$string['privacy:metadata:notifications:timecreated'] = 'The date at which the notification was created';
+$string['privacy:metadata:notifications:timeread'] = 'The time when the notification was read';
+$string['privacy:metadata:notifications:timecreated'] = 'The time when the notification was created';
 $string['privacy:metadata:notifications:useridfrom'] = 'The ID of the user who sent the notification';
 $string['privacy:metadata:notifications:useridto'] = 'The ID of the user who received the notification';
+$string['privacy:metadata:notifications:customdata'] = 'Custom data, usually containing internal IDs and a public URL of the sender picture (if any)';
 $string['privacy:metadata:preference:core_message_settings'] = 'Settings related to messaging';
 $string['privacy:request:preference:set'] = 'The value of the setting \'{$a->name}\' was \'{$a->value}\'';
+$string['privacy:export:conversationprefix'] = 'Conversation: ';
 $string['processorsettings'] = 'Processor settings';
 $string['removecontact'] = 'Remove contact';
+$string['removecontactconfirm'] = 'Are you sure you want to remove {$a} from your contacts?';
 $string['removecoursefilter'] = 'Remove filter for course {$a}';
-$string['removefromyourcontacts'] = 'Remove from your contacts';
+$string['removefromyourcontacts'] = 'Remove from contacts';
+$string['removefromfavourites'] = 'Unstar conversation';
+$string['requirecontacttomessage'] = 'You need to request {$a} to add you as a contact to be able to message them.';
 $string['requiresconfiguration'] = 'Requires configuration';
 $string['searchforuser'] = 'Search for a user';
 $string['searchforuserorcourse'] = 'Search for a user or course';
@@ -149,9 +226,13 @@ $string['searchcombined'] = 'Search people and messages';
 $string['seeall'] = 'See all';
 $string['selectmessagestodelete'] = 'Select messages to delete';
 $string['selectnotificationtoview'] = 'Select from the list of notifications on the side to view more details';
+$string['selfconversation'] = 'Personal space';
+$string['selfconversationdefaultmessage'] = 'Save draft messages, links, notes etc. to access later.';
 $string['send'] = 'Send';
+$string['sender'] = '{$a}:';
 $string['sendingvia'] = 'Sending "{$a->provider}" via "{$a->processor}"';
 $string['sendingviawhen'] = 'Sending "{$a->provider}" via "{$a->processor}" when {$a->state}';
+$string['sendcontactrequest'] = 'Send contact request';
 $string['sendmessage'] = 'Send message';
 $string['sendbulkmessage'] = 'Send message to {$a} people';
 $string['sendbulkmessagesingle'] = 'Send message to 1 person';
@@ -165,39 +246,52 @@ $string['showmessagewindowwithcount'] = 'Show message window with {$a} new messa
 $string['shownotificationwindownonew'] = 'Show notification window with no new notifications';
 $string['shownotificationwindowwithcount'] = 'Show notification window with {$a} new notifications';
 $string['togglenotificationmenu'] = 'Toggle notifications menu';
-$string['togglemessagemenu'] = 'Toggle messages menu';
-$string['touserdoesntexist'] = 'You can not send a message to a user id ({$a}) that doesn\'t exist';
+$string['togglemessagemenu'] = 'Toggle messaging drawer';
+$string['totalconversations'] = '{$a} total conversations';
+$string['touserdoesntexist'] = 'You cannot send a message to a user ID ({$a}) that doesn\'t exist.';
+$string['unabletomessage'] = 'You are unable to message this user';
+$string['unblock'] = 'Unblock';
 $string['unblockcontact'] = 'Unblock contact';
+$string['unblockuser'] = 'Unblock user';
+$string['unblockuserconfirm'] = 'Are you sure you want to unblock {$a}?';
 $string['unknownuser'] = 'Unknown user';
+$string['unmuteconversation'] = 'Unmute';
+$string['unreadconversations'] = 'There are {$a} unread conversations';
+$string['unreadmessages'] = 'There are {$a} unread messages';
 $string['unreadnotification'] = 'Unread notification: {$a}';
+$string['unreadnotifications'] = 'There are {$a} unread notifications';
+$string['unreadnewgroupconversationmessage'] = 'New message from {$a->name} in {$a->conversationname}';
 $string['unreadnewmessage'] = 'New message from {$a}';
+$string['useentertosend'] = 'Use enter to send';
+$string['usercantbemessaged'] = 'You can\'t message {$a} due to their message preferences. Try adding them as a contact.';
 $string['userisblockingyou'] = 'This user has blocked you from sending messages to them';
 $string['userisblockingyounoncontact'] = '{$a} only accepts messages from their contacts.';
+$string['userwouldliketocontactyou'] = '{$a} would like to contact you';
 $string['viewfullnotification'] = 'View full notification';
-$string['viewinganotherusersmessagearea'] = 'You are viewing another user\'s message area.';
 $string['viewmessageswith'] = 'View messages with {$a}';
 $string['viewnotificationresource'] = 'Go to: {$a}';
 $string['viewunreadmessageswith'] = 'View unread messages with {$a}';
 $string['writeamessage'] = 'Write a message...';
+$string['wouldliketocontactyou'] = 'Would like to contact you';
 $string['you'] = 'You:';
+$string['youhaveblockeduser'] = 'You have blocked this user.';
+$string['yourcontactrequestpending'] = 'Your contact request is pending with {$a}';
 
-// Deprecated since Moodle 3.2.
-$string['context'] = 'context';
-$string['discussion'] = 'Discussion';
-$string['emptysearchstring'] = 'You must search for something';
-$string['formorethan'] = 'For more than';
-$string['keywords'] = 'Keywords';
-$string['messagehistory'] = 'Message history';
-$string['newsearch'] = 'New search';
-$string['nosearchresults'] = 'There were no results from your search';
-$string['onlymycourses'] = 'Only in my courses';
-$string['pagerefreshes'] = 'This page refreshes automatically every {$a} seconds';
-$string['page-message-x'] = 'Any message pages';
-$string['recent'] = 'Recent';
-$string['savemysettings'] = 'Save my settings';
-$string['search'] = 'Search';
-$string['settingssaved'] = 'Your settings have been saved';
-$string['strftimedaydatetime'] = '%A, %d %B %Y, %I:%M %p';
-$string['timenosee'] = 'Minutes since I was last seen online';
-$string['timesent'] = 'Time sent';
-$string['userssearchresults'] = 'Users found: {$a}';
+// Deprecated since Moodle 3.6.
+$string['eventmessagecontactblocked'] = 'Message contact blocked';
+$string['eventmessagecontactunblocked'] = 'Message contact unblocked';
+$string['messagingdisabled'] = 'Messaging is disabled on this site, emails will be sent instead';
+$string['userisblockingyou'] = 'This user has blocked you from sending messages to them.';
+$string['userisblockingyounoncontact'] = '{$a} only accepts messages from their contacts.';
+
+// Deprecated since Moodle 3.7.
+$string['nomessages'] = 'No messages';
+$string['outputdisabled'] = 'Output disabled';
+$string['outputdoesnotexist'] = 'Message output does not exist';
+$string['outputenabled'] = 'Output enabled';
+$string['outputnotconfigured'] = 'Not configured';
+$string['canceledit'] = 'Cancel editing messages';
+
+// Deprecated since Moodle 3.9.
+$string['messagecontactrequestsnotification'] = '{$a} is requesting to be added as a contact.';
+$string['messagecontactrequestsnotificationsubject'] = 'Contact request from {$a}';
