@@ -84,6 +84,36 @@ function behat_error($errorcode, $text = '') {
     testing_error($errorcode, $text);
 }
 
+/**
+ * Return logical error string.
+ *
+ * @param int $errtype php error type.
+ * @return string string which will be returned.
+ */
+function behat_get_error_string($errtype) {
+    switch ($errtype) {
+        case E_USER_ERROR:
+            $errnostr = 'Fatal error';
+            break;
+        case E_WARNING:
+        case E_USER_WARNING:
+            $errnostr = 'Warning';
+            break;
+        case E_NOTICE:
+        case E_USER_NOTICE:
+        case E_STRICT:
+            $errnostr = 'Notice';
+            break;
+        case E_RECOVERABLE_ERROR:
+            $errnostr = 'Catchable';
+            break;
+        default:
+            $errnostr = 'Unknown error type';
+    }
+
+    return $errnostr;
+}
+
 /**
  * PHP errors handler to use when running behat tests.
  *
@@ -108,7 +138,7 @@ function behat_error_handler($errno, $errstr, $errfile, $errline, $errcontext) {
     // set to DEVELOPER and will always include E_NOTICE,E_USER_NOTICE... as part of E_ALL, if the current
     // error_reporting() value does not include one of those levels is because it has been forced through
     // the moodle code (see fix_utf8() for example) in that cases we respect the forced error level value.
-    $respect = array(E_NOTICE, E_USER_NOTICE, E_STRICT, E_WARNING, E_USER_WARNING);
+    $respect = array(E_NOTICE, E_USER_NOTICE, E_STRICT, E_WARNING, E_USER_WARNING, E_DEPRECATED, E_USER_DEPRECATED);
     foreach ($respect as $respectable) {
 
         // If the current value does not include this kind of errors and the reported error is
@@ -121,35 +151,59 @@ function behat_error_handler($errno, $errstr, $errfile, $errline, $errcontext) {
     // Using the default one in case there is a fatal catchable error.
     default_error_handler($errno, $errstr, $errfile, $errline, $errcontext);
 
-    switch ($errno) {
-        case E_USER_ERROR:
-            $errnostr = 'Fatal error';
-            break;
-        case E_WARNING:
-        case E_USER_WARNING:
-            $errnostr = 'Warning';
-            break;
-        case E_NOTICE:
-        case E_USER_NOTICE:
-        case E_STRICT:
-            $errnostr = 'Notice';
-            break;
-        case E_RECOVERABLE_ERROR:
-            $errnostr = 'Catchable';
-            break;
-        default:
-            $errnostr = 'Unknown error type';
-    }
+    $errnostr = behat_get_error_string($errno);
 
-    // Wrapping the output.
-    echo '<div class="phpdebugmessage" data-rel="phpdebugmessage">' . PHP_EOL;
-    echo "$errnostr: $errstr in $errfile on line $errline" . PHP_EOL;
-    echo '</div>';
+    // If ajax script then throw exception, so the calling api catch it and show it on web page.
+    if (defined('AJAX_SCRIPT')) {
+        throw new Exception("$errnostr: $errstr in $errfile on line $errline");
+    } else {
+        // Wrapping the output.
+        echo '<div class="phpdebugmessage" data-rel="phpdebugmessage">' . PHP_EOL;
+        echo "$errnostr: $errstr in $errfile on line $errline" . PHP_EOL;
+        echo '</div>';
+    }
 
     // Also use the internal error handler so we keep the usual behaviour.
     return false;
 }
 
+/**
+ * Before shutdown save last error entries, so we can fail the test.
+ */
+function behat_shutdown_function() {
+    // If any error found, then save it.
+    if ($error = error_get_last()) {
+        // Ignore E_WARNING, as they might come via ( @ )suppression and might lead to false failure.
+        if (isset($error['type']) && !($error['type'] & E_WARNING)) {
+
+            $errors = behat_get_shutdown_process_errors();
+
+            $errors[] = $error;
+            $errorstosave = json_encode($errors);
+
+            set_config('process_errors', $errorstosave, 'tool_behat');
+        }
+    }
+}
+
+/**
+ * Return php errors save which were save during shutdown.
+ *
+ * @return array
+ */
+function behat_get_shutdown_process_errors() {
+    global $DB;
+
+    // Don't use get_config, as it use cache and return invalid value, between selenium and cli process.
+    $phperrors = $DB->get_field('config_plugins', 'value', array('name' => 'process_errors', 'plugin' => 'tool_behat'));
+
+    if (!empty($phperrors)) {
+        return json_decode($phperrors, true);
+    } else {
+        return array();
+    }
+}
+
 /**
  * Restrict the config.php settings allowed.
  *
@@ -165,8 +219,8 @@ function behat_clean_init_config() {
         'wwwroot', 'dataroot', 'dirroot', 'admin', 'directorypermissions', 'filepermissions',
         'umaskpermissions', 'dbtype', 'dblibrary', 'dbhost', 'dbname', 'dbuser', 'dbpass', 'prefix',
         'dboptions', 'proxyhost', 'proxyport', 'proxytype', 'proxyuser', 'proxypassword',
-        'proxybypass', 'theme', 'pathtogs', 'pathtoclam', 'pathtodu', 'aspellpath', 'pathtodot', 'skiplangupgrade',
-        'altcacheconfigpath'
+        'proxybypass', 'pathtogs', 'pathtophp', 'pathtodu', 'aspellpath', 'pathtodot', 'skiplangupgrade',
+        'altcacheconfigpath', 'pathtounoconv', 'alternative_file_system_class', 'pathtopython'
     ));
 
     // Add extra allowed settings.
@@ -180,15 +234,6 @@ function behat_clean_init_config() {
             unset($CFG->{$key});
         }
     }
-
-    // Here we are forcing the navbar to be absolutely positioned in Chrome, Safari and IE in order to
-    // avoid a driver bug whereby when the browser scrolls something into view it doesn't account
-    // for fixed positioned elements that end up obscuring the item thus leading to errors that
-    // could be avoided by scrolling an additional amount.
-    // This should be removed as soon as the affected drivers have been fixed.
-    $CFG->forced_plugin_settings['theme_clean'] = array(
-        'customcss' => 'body.safari .navbar-fixed-top, body.ie .navbar-fixed-top {position: absolute;}'
-    );
 }
 
 /**
@@ -229,6 +274,33 @@ function behat_check_config_vars() {
             'Define $CFG->behat_dataroot in config.php');
     }
     clearstatcache();
+    if (!file_exists($CFG->behat_dataroot_parent)) {
+        $permissions = isset($CFG->directorypermissions) ? $CFG->directorypermissions : 02777;
+        umask(0);
+        if (!mkdir($CFG->behat_dataroot_parent, $permissions, true)) {
+            behat_error(BEHAT_EXITCODE_PERMISSIONS, '$CFG->behat_dataroot directory can not be created');
+        }
+    }
+    $CFG->behat_dataroot_parent = realpath($CFG->behat_dataroot_parent);
+    if (empty($CFG->behat_dataroot_parent) or !is_dir($CFG->behat_dataroot_parent) or !is_writable($CFG->behat_dataroot_parent)) {
+        behat_error(BEHAT_EXITCODE_CONFIG,
+            '$CFG->behat_dataroot in config.php must point to an existing writable directory');
+    }
+    if (!empty($CFG->dataroot) and $CFG->behat_dataroot_parent == realpath($CFG->dataroot)) {
+        behat_error(BEHAT_EXITCODE_CONFIG,
+            '$CFG->behat_dataroot in config.php must be different from $CFG->dataroot');
+    }
+    if (!empty($CFG->phpunit_dataroot) and $CFG->behat_dataroot_parent == realpath($CFG->phpunit_dataroot)) {
+        behat_error(BEHAT_EXITCODE_CONFIG,
+            '$CFG->behat_dataroot in config.php must be different from $CFG->phpunit_dataroot');
+    }
+
+    // This request is coming from admin/tool/behat/cli/util.php which will call util_single.php. So just return from
+    // here as we don't need to create a dataroot for single run.
+    if (defined('BEHAT_PARALLEL_UTIL') && BEHAT_PARALLEL_UTIL && empty($CFG->behatrunprocess)) {
+        return;
+    }
+
     if (!file_exists($CFG->behat_dataroot)) {
         $permissions = isset($CFG->directorypermissions) ? $CFG->directorypermissions : 02777;
         umask(0);
@@ -237,18 +309,6 @@ function behat_check_config_vars() {
         }
     }
     $CFG->behat_dataroot = realpath($CFG->behat_dataroot);
-    if (empty($CFG->behat_dataroot) or !is_dir($CFG->behat_dataroot) or !is_writable($CFG->behat_dataroot)) {
-        behat_error(BEHAT_EXITCODE_CONFIG,
-            '$CFG->behat_dataroot in config.php must point to an existing writable directory');
-    }
-    if (!empty($CFG->dataroot) and $CFG->behat_dataroot == realpath($CFG->dataroot)) {
-        behat_error(BEHAT_EXITCODE_CONFIG,
-            '$CFG->behat_dataroot in config.php must be different from $CFG->dataroot');
-    }
-    if (!empty($CFG->phpunit_dataroot) and $CFG->behat_dataroot == realpath($CFG->phpunit_dataroot)) {
-        behat_error(BEHAT_EXITCODE_CONFIG,
-            '$CFG->behat_dataroot in config.php must be different from $CFG->phpunit_dataroot');
-    }
 }
 
 /**
@@ -291,6 +351,10 @@ function behat_update_vars_for_process() {
     $behatrunprocess = behat_get_run_process();
     $CFG->behatrunprocess = $behatrunprocess;
 
+    // Data directory will be a directory under parent directory.
+    $CFG->behat_dataroot_parent = $CFG->behat_dataroot;
+    $CFG->behat_dataroot .= '/'. BEHAT_PARALLEL_SITE_NAME;
+
     if ($behatrunprocess) {
         if (empty($CFG->behat_parallel_run[$behatrunprocess - 1]['behat_wwwroot'])) {
             // Set www root for run process.
@@ -397,26 +461,40 @@ function behat_get_run_process() {
             }
         }
     } else if (defined('BEHAT_TEST') || defined('BEHAT_UTIL')) {
-        if ($match = preg_filter('#--run=(.+)#', '$1', $argv)) {
-            $behatrunprocess = reset($match);
-        } else if ($k = array_search('--config', $argv)) {
-            $behatconfig = str_replace("\\", "/", $argv[$k + 1]);
-            // Try get it from config if present.
-            if (!empty($CFG->behat_parallel_run)) {
-                foreach ($CFG->behat_parallel_run as $run => $parallelconfig) {
-                    if (!empty($parallelconfig['behat_dataroot']) &&
-                        $parallelconfig['behat_dataroot'] . '/behat/behat.yml' == $behatconfig) {
+        $behatconfig = '';
 
-                        $behatrunprocess = $run + 1; // We start process from 1.
-                        break;
+        if ($match = preg_filter('#--run=(.+)#', '$1', $argv)) {
+            // Try to guess the run from the existence of the --run arg.
+            $behatrunprocess = reset($match);
+
+        } else {
+            // Try to guess the run from the existence of the --config arg. Note there are 2 alternatives below.
+            if ($k = array_search('--config', $argv)) {
+                // Alternative 1: --config /path/to/config.yml => (next arg, pick it).
+                $behatconfig = str_replace("\\", "/", $argv[$k + 1]);
+
+            } else if ($config = preg_filter('#^(?:--config[ =]*)(.+)$#', '$1', $argv)) {
+                // Alternative 2: --config=/path/to/config.yml => (same arg, just get the path part).
+                $behatconfig = str_replace("\\", "/", reset($config));
+            }
+
+            // Try get it from config if present.
+            if ($behatconfig) {
+                if (!empty($CFG->behat_parallel_run)) {
+                    foreach ($CFG->behat_parallel_run as $run => $parallelconfig) {
+                        if (!empty($parallelconfig['behat_dataroot']) &&
+                                $parallelconfig['behat_dataroot'] . '/behat/behat.yml' == $behatconfig) {
+                            $behatrunprocess = $run + 1; // We start process from 1.
+                            break;
+                        }
                     }
                 }
-            }
-            // Check if default behat datroot increment was done.
-            if (empty($behatrunprocess)) {
-                $behatdataroot = str_replace("\\", "/", $CFG->behat_dataroot);
-                $behatrunprocess = preg_filter("#^{$behatdataroot}" . "(.+?)[/|\\\]behat[/|\\\]behat\.yml#", '$1',
-                    $behatconfig);
+                // Check if default behat dataroot increment was done.
+                if (empty($behatrunprocess)) {
+                    $behatdataroot = str_replace("\\", "/", $CFG->behat_dataroot . '/' . BEHAT_PARALLEL_SITE_NAME);
+                    $behatrunprocess = preg_filter("#^{$behatdataroot}" . "(.+?)[/|\\\]behat[/|\\\]behat\.yml#", '$1',
+                        $behatconfig);
+                }
             }
         }
     }
@@ -429,16 +507,24 @@ function behat_get_run_process() {
  *
  * @param array $cmds list of commands to be executed.
  * @param string $cwd absolute path of working directory.
+ * @param int $delay time in seconds to add delay between each parallel process.
  * @return array list of processes.
  */
-function cli_execute_parallel($cmds, $cwd = null) {
+function cli_execute_parallel($cmds, $cwd = null, $delay = 0) {
     require_once(__DIR__ . "/../../vendor/autoload.php");
 
     $processes = array();
 
     // Create child process.
     foreach ($cmds as $name => $cmd) {
-        $process = new Symfony\Component\Process\Process($cmd);
+        if (method_exists('\\Symfony\\Component\\Process\\Process', 'fromShellCommandline')) {
+            // Process 4.2 and up.
+            $process = Symfony\Component\Process\Process::fromShellCommandline($cmd);
+        } else {
+            // Process 4.1 and older.
+            $process = new Symfony\Component\Process\Process(null);
+            $process->setCommandLine($cmd);
+        }
 
         $process->setWorkingDirectory($cwd);
         $process->setTimeout(null);
@@ -455,6 +541,11 @@ function cli_execute_parallel($cmds, $cwd = null) {
             }
             exit(1);
         }
+
+        // Sleep for specified delay.
+        if ($delay) {
+            sleep($delay);
+        }
     }
     return $processes;
 }
