@@ -64,6 +64,11 @@ abstract class testing_util {
      */
     private static $tablesequences = array();
 
+    /**
+     * @var array list of updated tables.
+     */
+    public static $tableupdated = array();
+
     /**
      * @var array original structure of all database tables
      */
@@ -181,7 +186,7 @@ abstract class testing_util {
      * @return bool
      */
     public static function is_test_data_updated() {
-        global $CFG;
+        global $DB;
 
         $framework = self::get_framework();
 
@@ -201,7 +206,8 @@ abstract class testing_util {
             return false;
         }
 
-        $dbhash = get_config('core', $framework . 'test');
+        // A direct database request must be used to avoid any possible caching of an older value.
+        $dbhash = $DB->get_field('config', 'value', array('name' => $framework . 'test'));
         if ($hash !== $dbhash) {
             return false;
         }
@@ -364,11 +370,9 @@ abstract class testing_util {
                     continue;
                 }
 
-                if (!is_null($info->auto_increment)) {
+                if (!is_null($info->auto_increment) && $info->rows == 0 && ($info->auto_increment == 1)) {
                     $table = preg_replace('/^'.preg_quote($prefix, '/').'/', '', $table);
-                    if (isset(self::$tablesequences[$table]) && ($info->auto_increment == self::$tablesequences[$table])) {
-                        $empties[$table] = $table;
-                    }
+                    $empties[$table] = $table;
                 }
             }
             $rs->close();
@@ -464,6 +468,8 @@ abstract class testing_util {
             return;
         }
 
+        $updatedtables = self::$tableupdated;
+
         // If all starting Id's are the same, it's difficult to detect coding and testing
         // errors that use the incorrect id in tests.  The classic case is cmid vs instance id.
         // To reduce the chance of the coding error, we start sequences at different values where possible.
@@ -480,6 +486,10 @@ abstract class testing_util {
             $queries = array();
             $prefix = $DB->get_prefix();
             foreach ($data as $table => $records) {
+                // If table is not modified then no need to do anything.
+                if (!isset($updatedtables[$table])) {
+                    continue;
+                }
                 if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
                     $nextid = self::get_next_sequence_starting_value($records, $table);
                     $queries[] = "ALTER SEQUENCE {$prefix}{$table}_id_seq RESTART WITH $nextid";
@@ -490,6 +500,7 @@ abstract class testing_util {
             }
 
         } else if ($dbfamily === 'mysql') {
+            $queries = array();
             $sequences = array();
             $prefix = $DB->get_prefix();
             $rs = $DB->get_recordset_sql("SHOW TABLE STATUS LIKE ?", array($prefix.'%'));
@@ -507,11 +518,15 @@ abstract class testing_util {
             $rs->close();
             $prefix = $DB->get_prefix();
             foreach ($data as $table => $records) {
+                // If table is not modified then no need to do anything.
+                if (!isset($updatedtables[$table])) {
+                    continue;
+                }
                 if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
                     if (isset($sequences[$table])) {
                         $nextid = self::get_next_sequence_starting_value($records, $table);
                         if ($sequences[$table] != $nextid) {
-                            $DB->change_database_structure("ALTER TABLE {$prefix}{$table} AUTO_INCREMENT = $nextid");
+                            $queries[] = "ALTER TABLE {$prefix}{$table} AUTO_INCREMENT = $nextid";
                         }
                     } else {
                         // some problem exists, fallback to standard code
@@ -519,6 +534,9 @@ abstract class testing_util {
                     }
                 }
             }
+            if ($queries) {
+                $DB->change_database_structure(implode(';', $queries));
+            }
 
         } else if ($dbfamily === 'oracle') {
             $sequences = self::get_sequencenames();
@@ -536,6 +554,10 @@ abstract class testing_util {
             $rs->close();
 
             foreach ($data as $table => $records) {
+                // If table is not modified then no need to do anything.
+                if (!isset($updatedtables[$table])) {
+                    continue;
+                }
                 if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
                     $lastrecord = end($records);
                     if ($lastrecord) {
@@ -559,11 +581,12 @@ abstract class testing_util {
         } else {
             // note: does mssql support any kind of faster reset?
             // This also implies mssql will not use unique sequence values.
-            if (is_null($empties)) {
+            if (is_null($empties) and (empty($updatedtables))) {
                 $empties = self::guess_unmodified_empty_tables();
             }
             foreach ($data as $table => $records) {
-                if (isset($empties[$table])) {
+                // If table is not modified then no need to do anything.
+                if (isset($empties[$table]) or (!isset($updatedtables[$table]))) {
                     continue;
                 }
                 if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
@@ -596,25 +619,43 @@ abstract class testing_util {
             return false;
         }
 
-        $empties = self::guess_unmodified_empty_tables();
+        $empties = array();
+        // Use local copy of self::$tableupdated, as list gets updated in for loop.
+        $updatedtables = self::$tableupdated;
+
+        // If empty tablesequences list then it's the very first run.
+        if (empty(self::$tablesequences) && (($DB->get_dbfamily() != 'mysql') && ($DB->get_dbfamily() != 'postgres'))) {
+            // Only Mysql and Postgres support random sequence, so don't guess, just reset everything on very first run.
+            $empties = self::guess_unmodified_empty_tables();
+        }
+
+        // Check if any table has been modified by behat selenium process.
+        if (defined('BEHAT_SITE_RUNNING')) {
+            // Crazy way to reset :(.
+            $tablesupdatedfile = self::get_tables_updated_by_scenario_list_path();
+            if ($tablesupdated = @json_decode(file_get_contents($tablesupdatedfile), true)) {
+                self::$tableupdated = array_merge(self::$tableupdated, $tablesupdated);
+                unlink($tablesupdatedfile);
+            }
+            $updatedtables = self::$tableupdated;
+        }
 
         $borkedmysql = false;
         if ($DB->get_dbfamily() === 'mysql') {
             $version = $DB->get_server_info();
-            if (version_compare($version['version'], '5.6.0') == 1 and version_compare($version['version'], '5.6.16') == -1) {
-                // Everything that comes from Oracle is evil!
-                //
-                // See http://dev.mysql.com/doc/refman/5.6/en/alter-table.html
-                // You cannot reset the counter to a value less than or equal to to the value that is currently in use.
-                //
-                // From 5.6.16 release notes:
-                //   InnoDB: The ALTER TABLE INPLACE algorithm would fail to decrease the auto-increment value.
-                //           (Bug #17250787, Bug #69882)
+            // Everything that comes from Oracle is evil!
+            //
+            // See http://dev.mysql.com/doc/refman/5.6/en/alter-table.html
+            // You cannot reset the counter to a value less than or equal to to the value that is currently in use.
+            //
+            // From 5.6.16 release notes:
+            //   InnoDB: The ALTER TABLE INPLACE algorithm would fail to decrease the auto-increment value.
+            //           (Bug #17250787, Bug #69882)
+            if (version_compare($version['version'], '5.6.0', '>=') && version_compare($version['version'], '5.6.16', '<')) {
                 $borkedmysql = true;
+            }
 
-            } else if (version_compare($version['version'], '10.0.0') == 1) {
-                // And MariaDB is no better!
-                // Let's hope they pick the patch sometime later...
+            if (version_compare($version['version'], '5.7.0', '>=') && version_compare($version['version'], '5.7.4', '<')) {
                 $borkedmysql = true;
             }
         }
@@ -634,9 +675,16 @@ abstract class testing_util {
                     $mysqlsequences[$table] = $info->auto_increment;
                 }
             }
+            $rs->close();
         }
 
         foreach ($data as $table => $records) {
+            // If table is not modified then no need to do anything.
+            // $updatedtables tables is set after the first run, so check before checking for specific table update.
+            if (!empty($updatedtables) && !isset($updatedtables[$table])) {
+                continue;
+            }
+
             if ($borkedmysql) {
                 if (empty($records)) {
                     if (!isset($empties[$table])) {
@@ -712,6 +760,8 @@ abstract class testing_util {
             }
         }
 
+        self::reset_updated_table_list();
+
         return true;
     }
 
@@ -762,14 +812,16 @@ abstract class testing_util {
         }
 
         make_temp_directory('');
+        make_backup_temp_directory('');
         make_cache_directory('');
         make_localcache_directory('');
+        // Purge all data from the caches. This is required for consistency between tests.
+        // Any file caches that happened to be within the data root will have already been clearer (because we just deleted cache)
+        // and now we will purge any other caches as well.  This must be done before the cache_factory::reset() as that
+        // removes all definitions of caches and purge does not have valid caches to operate on.
+        cache_helper::purge_all();
         // Reset the cache API so that it recreates it's required directories as well.
         cache_factory::reset();
-        // Purge all data from the caches. This is required for consistency.
-        // Any file caches that happened to be within the data root will have already been clearer (because we just deleted cache)
-        // and now we will purge any other caches as well.
-        cache_helper::purge_all();
     }
 
     /**
@@ -854,6 +906,63 @@ abstract class testing_util {
         return $hash;
     }
 
+    /**
+     * Set state of modified tables.
+     *
+     * @param string $sql sql which is updating the table.
+     */
+    public static function set_table_modified_by_sql($sql) {
+        global $DB;
+
+        $prefix = $DB->get_prefix();
+
+        preg_match('/( ' . $prefix . '\w*)(.*)/', $sql, $matches);
+        // Ignore random sql for testing like "XXUPDATE SET XSSD".
+        if (!empty($matches[1])) {
+            $table = trim($matches[1]);
+            $table = preg_replace('/^' . preg_quote($prefix, '/') . '/', '', $table);
+            self::$tableupdated[$table] = true;
+
+            if (defined('BEHAT_SITE_RUNNING')) {
+                $tablesupdatedfile = self::get_tables_updated_by_scenario_list_path();
+                $tablesupdated = @json_decode(file_get_contents($tablesupdatedfile), true);
+                if (!isset($tablesupdated[$table])) {
+                    $tablesupdated[$table] = true;
+                    @file_put_contents($tablesupdatedfile, json_encode($tablesupdated, JSON_PRETTY_PRINT));
+                }
+            }
+        }
+    }
+
+    /**
+     * Reset updated table list. This should be done after every reset.
+     */
+    public static function reset_updated_table_list() {
+        self::$tableupdated = array();
+    }
+
+    /**
+     * Delete tablesupdatedbyscenario file. This should be called before suite,
+     * to ensure full db reset.
+     */
+    public static function clean_tables_updated_by_scenario_list() {
+        $tablesupdatedfile = self::get_tables_updated_by_scenario_list_path();
+        if (file_exists($tablesupdatedfile)) {
+            unlink($tablesupdatedfile);
+        }
+
+        // Reset static cache of cli process.
+        self::reset_updated_table_list();
+    }
+
+    /**
+     * Returns the path to the file which holds list of tables updated in scenario.
+     * @return string
+     */
+    protected final static function get_tables_updated_by_scenario_list_path() {
+        return self::get_dataroot() . '/tablesupdatedbyscenario.json';
+    }
+
     /**
      * Drop the whole test database
      * @static
