@@ -36,6 +36,9 @@ require_once($CFG->dirroot.'/grade/grading/lib.php');
  */
 class assign_feedback_offline extends assign_feedback_plugin {
 
+    /** @var boolean|null $enabledcache Cached lookup of the is_enabled function */
+    private $enabledcache = null;
+
     /**
      * Get the name of the file feedback plugin
      * @return string
@@ -64,21 +67,34 @@ class assign_feedback_offline extends assign_feedback_plugin {
         return true;
     }
 
+    /**
+     * This plugin does not save through the normal interface so this returns false.
+     *
+     * @param stdClass $grade The grade.
+     * @param stdClass $data Form data from the feedback form.
+     * @return boolean - False
+     */
+    public function is_feedback_modified(stdClass $grade, stdClass $data) {
+        return false;
+    }
+
     /**
      * Loop through uploaded grades and update the grades for this assignment
      *
      * @param int $draftid - The unique draft item id for this import
      * @param int $importid - The unique import ID for this csv import operation
      * @param bool $ignoremodified - Ignore the last modified date when checking fields
+     * @param string $encoding - Encoding of the file being processed.
+     * @param string $separator - The character used to separate the information.
      * @return string - The html response
      */
-    public function process_import_grades($draftid, $importid, $ignoremodified) {
+    public function process_import_grades($draftid, $importid, $ignoremodified, $encoding = 'utf-8', $separator = 'comma') {
         global $USER, $DB;
 
         require_sesskey();
         require_capability('mod/assign:grade', $this->assignment->get_context());
 
-        $gradeimporter = new assignfeedback_offline_grade_importer($importid, $this->assignment);
+        $gradeimporter = new assignfeedback_offline_grade_importer($importid, $this->assignment, $encoding, $separator);
 
         $context = context_user::instance($USER->id);
         $fs = get_file_storage();
@@ -100,23 +116,23 @@ class assign_feedback_offline extends assign_feedback_plugin {
                                                                      'pluginsubtype'=>'assignfeedback',
                                                                      'plugin'=>'offline',
                                                                      'pluginaction'=>'uploadgrades',
-                                                                     'id'=>$assignment->get_course_module()->id));
+                                                                     'id' => $this->assignment->get_course_module()->id));
             print_error('invalidgradeimport', 'assignfeedback_offline', $thisurl);
             return;
         }
         // Does this assignment use a scale?
         $scaleoptions = null;
         if ($this->assignment->get_instance()->grade < 0) {
-            $scale = $DB->get_record('scale', array('id'=>-($this->assignment->get_instance()->grade)));
-            if ($scale) {
-                $scaleoptions = explode(',', $scale->scale);
+            if ($scale = $DB->get_record('scale', array('id'=>-($this->assignment->get_instance()->grade)))) {
+                $scaleoptions = make_menu_from_list($scale->scale);
             }
         }
         // We may need to upgrade the gradebook comments after this update.
         $adminconfig = $this->assignment->get_admin_config();
         $gradebookplugin = $adminconfig->feedback_plugin_for_gradebook;
 
-        $updatecount = 0;
+        $updategradecount = 0;
+        $updatefeedbackcount = 0;
         while ($record = $gradeimporter->next()) {
             $user = $record->user;
             $modified = $record->modified;
@@ -127,7 +143,7 @@ class assign_feedback_offline extends assign_feedback_plugin {
                 // This is a scale - we need to convert any grades to indexes in the scale.
                 $scaleindex = array_search($record->grade, $scaleoptions);
                 if ($scaleindex !== false) {
-                    $record->grade = $scaleindex + 1;
+                    $record->grade = $scaleindex;
                 } else {
                     $record->grade = '';
                 }
@@ -164,8 +180,7 @@ class assign_feedback_offline extends assign_feedback_plugin {
                 $grade->grader = $USER->id;
                 if ($this->assignment->update_grade($grade)) {
                     $this->assignment->notify_grade_modified($grade);
-                    $this->assignment->add_to_log('grade submission', $this->assignment->format_grade_for_log($grade));
-                    $updatecount += 1;
+                    $updategradecount += 1;
                 }
             }
 
@@ -183,17 +198,10 @@ class assign_feedback_offline extends assign_feedback_plugin {
                         }
                     }
                     if ($newvalue != $oldvalue) {
-                        $updatecount += 1;
+                        $updatefeedbackcount += 1;
                         $grade = $this->assignment->get_user_grade($record->user->id, true);
                         $this->assignment->notify_grade_modified($grade);
-                        if ($plugin->set_editor_text($field, $newvalue, $grade->id)) {
-                            $logdesc = get_string('feedbackupdate', 'assignfeedback_offline',
-                                                  array('field'=>$description,
-                                                        'student'=>$userdesc,
-                                                        'text'=>$newvalue));
-
-                            $this->assignment->add_to_log('save grading feedback', $logdesc);
-                        }
+                        $plugin->set_editor_text($field, $newvalue, $grade->id);
 
                         // If this is the gradebook comments plugin - post an update to the gradebook.
                         if (($plugin->get_subtype() . '_' . $plugin->get_type()) == $gradebookplugin) {
@@ -215,7 +223,11 @@ class assign_feedback_offline extends assign_feedback_plugin {
                                                   false,
                                                   $this->assignment->get_course_module()->id,
                                                   get_string('importgrades', 'assignfeedback_offline')));
-        $o .= $renderer->box(get_string('updatedgrades', 'assignfeedback_offline', $updatecount));
+        $strparams = [
+            'gradeupdatescount' => $updategradecount,
+            'feedbackupdatescount' => $updatefeedbackcount,
+        ];
+        $o .= $renderer->box(get_string('updatedgrades', 'assignfeedback_offline', $strparams));
         $url = new moodle_url('view.php',
                               array('id'=>$this->assignment->get_course_module()->id,
                                     'action'=>'grading'));
@@ -256,7 +268,8 @@ class assign_feedback_offline extends assign_feedback_plugin {
                    ($csvdata = $mform->get_file_content('gradesfile'))) {
 
             $importid = csv_import_reader::get_new_iid('assignfeedback_offline');
-            $gradeimporter = new assignfeedback_offline_grade_importer($importid, $this->assignment);
+            $gradeimporter = new assignfeedback_offline_grade_importer($importid, $this->assignment,
+                    $data->encoding, $data->separator);
             // File exists and was valid.
             $ignoremodified = !empty($data->ignoremodified);
 
@@ -278,11 +291,12 @@ class assign_feedback_offline extends assign_feedback_plugin {
             $o .= $renderer->render(new assign_form('confirmimport', $mform));
             $o .= $renderer->render_footer();
         } else if ($confirm) {
-
             $importid = optional_param('importid', 0, PARAM_INT);
             $draftid = optional_param('draftid', 0, PARAM_INT);
+            $encoding = optional_param('encoding', 'utf-8', PARAM_ALPHANUMEXT);
+            $separator = optional_param('separator', 'comma', PARAM_ALPHA);
             $ignoremodified = optional_param('ignoremodified', 0, PARAM_BOOL);
-            $gradeimporter = new assignfeedback_offline_grade_importer($importid, $this->assignment);
+            $gradeimporter = new assignfeedback_offline_grade_importer($importid, $this->assignment, $encoding, $separator);
             $mform = new assignfeedback_offline_import_grades_form(null, array('assignment'=>$this->assignment,
                                                                        'csvdata'=>'',
                                                                        'ignoremodified'=>$ignoremodified,
@@ -295,7 +309,7 @@ class assign_feedback_offline extends assign_feedback_plugin {
                 return;
             }
 
-            $o .= $this->process_import_grades($draftid, $importid, $ignoremodified);
+            $o .= $this->process_import_grades($draftid, $importid, $ignoremodified, $encoding, $separator);
         } else {
 
             $o .= $renderer->render(new assign_header($this->assignment->get_instance(),
@@ -374,14 +388,18 @@ class assign_feedback_offline extends assign_feedback_plugin {
      * @return bool
      */
     public function is_enabled() {
-        $gradingmanager = get_grading_manager($this->assignment->get_context(), 'mod_assign', 'submissions');
-        $controller = $gradingmanager->get_active_controller();
-        $active = !empty($controller);
+        if ($this->enabledcache === null) {
+            $gradingmanager = get_grading_manager($this->assignment->get_context(), 'mod_assign', 'submissions');
+            $controller = $gradingmanager->get_active_controller();
+            $active = !empty($controller);
 
-        if ($active) {
-            return false;
+            if ($active) {
+                $this->enabledcache = false;
+            } else {
+                $this->enabledcache = parent::is_enabled();
+            }
         }
-        return parent::is_enabled();
+        return $this->enabledcache;
     }
 
     /**
@@ -393,4 +411,13 @@ class assign_feedback_offline extends assign_feedback_plugin {
         return false;
     }
 
+    /**
+     * Return the plugin configs for external functions.
+     *
+     * @return array the list of settings
+     * @since Moodle 3.2
+     */
+    public function get_config_for_external() {
+        return (array) $this->get_config();
+    }
 }
