@@ -22,6 +22,8 @@
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
+use mod_forum\grades\forum_gradeitem;
+
 require_once('../../config.php');
 
 $managerfactory = mod_forum\local\container::get_manager_factory();
@@ -76,12 +78,41 @@ $cm = \cm_info::create($coursemodule);
 
 require_course_login($course, true, $cm);
 
+$istypesingle = $forum->get_type() === 'single';
+$saveddisplaymode = get_user_preferences('forum_displaymode', $CFG->forum_displaymode);
+
+if ($mode) {
+    $displaymode = $mode;
+} else {
+    $displaymode = $saveddisplaymode;
+}
+
+if (get_user_preferences('forum_useexperimentalui', false)) {
+    if ($displaymode == FORUM_MODE_NESTED) {
+        $displaymode = FORUM_MODE_NESTED_V2;
+    }
+} else {
+    if ($displaymode == FORUM_MODE_NESTED_V2) {
+        $displaymode = FORUM_MODE_NESTED;
+    }
+}
+
+if ($displaymode != $saveddisplaymode) {
+    set_user_preference('forum_displaymode', $displaymode);
+}
+
 $PAGE->set_context($forum->get_context());
 $PAGE->set_title($forum->get_name());
 $PAGE->add_body_class('forumtype-' . $forum->get_type());
 $PAGE->set_heading($course->fullname);
 $PAGE->set_button(forum_search_form($course, $search));
 
+if ($istypesingle && $displaymode == FORUM_MODE_NESTED_V2) {
+    $PAGE->add_body_class('nested-v2-display-mode reset-style');
+    $settingstrigger = $OUTPUT->render_from_template('mod_forum/settings_drawer_trigger', null);
+    $PAGE->add_header_action($settingstrigger);
+}
+
 if (empty($cm->visible) && !has_capability('moodle/course:viewhiddenactivities', $forum->get_context())) {
     redirect(
         $urlfactory->get_course_url_from_forum($forum),
@@ -94,7 +125,7 @@ if (empty($cm->visible) && !has_capability('moodle/course:viewhiddenactivities',
 if (!$capabilitymanager->can_view_discussions($USER)) {
     redirect(
         $urlfactory->get_course_url_from_forum($forum),
-        get_string('noviewdiscussionspermission', 'fourm'),
+        get_string('noviewdiscussionspermission', 'forum'),
         null,
         \core\output\notification::NOTIFY_WARNING
     );
@@ -125,16 +156,10 @@ if (!empty($CFG->enablerssfeeds) && !empty($CFG->forum_enablerssfeeds) && $forum
 echo $OUTPUT->header();
 echo $OUTPUT->heading(format_string($forum->get_name()), 2);
 
-if ('single' !== $forum->get_type() && !empty($forum->get_intro())) {
+if (!$istypesingle && !empty($forum->get_intro())) {
     echo $OUTPUT->box(format_module_intro('forum', $forumrecord, $cm->id), 'generalbox', 'intro');
 }
 
-if ($mode) {
-    set_user_preference('forum_displaymode', $mode);
-}
-
-$displaymode = get_user_preferences('forum_displaymode', $CFG->forum_displaymode);
-
 if ($sortorder) {
     set_user_preference('forum_discussionlistsortorder', $sortorder);
 }
@@ -146,6 +171,42 @@ $groupid = groups_get_activity_group($cm, true) ?: null;
 $rendererfactory = mod_forum\local\container::get_renderer_factory();
 switch ($forum->get_type()) {
     case 'single':
+        $forumgradeitem = forum_gradeitem::load_from_forum_entity($forum);
+        if ($capabilitymanager->can_grade($USER)) {
+
+            if ($forumgradeitem->is_grading_enabled()) {
+                $groupid = groups_get_activity_group($cm, true) ?: null;
+                $gradeobj = (object) [
+                    'contextid' => $forum->get_context()->id,
+                    'cmid' => $cmid,
+                    'name' => format_string($forum->get_name()),
+                    'courseid' => $course->id,
+                    'coursename' => format_string($course->shortname),
+                    'experimentaldisplaymode' => $displaymode == FORUM_MODE_NESTED_V2,
+                    'groupid' => $groupid,
+                    'gradingcomponent' => $forumgradeitem->get_grading_component_name(),
+                    'gradingcomponentsubtype' => $forumgradeitem->get_grading_component_subtype(),
+                    'sendstudentnotifications' => $forum->should_notify_students_default_when_grade_for_forum(),
+                ];
+                echo $OUTPUT->render_from_template('mod_forum/grades/grade_button', $gradeobj);
+            }
+        } else {
+            if ($forumgradeitem->is_grading_enabled()) {
+                $groupid = groups_get_activity_group($cm, true) ?: null;
+                $gradeobj = (object) [
+                    'contextid' => $forum->get_context()->id,
+                    'cmid' => $cmid,
+                    'name' => format_string($forum->get_name()),
+                    'courseid' => $course->id,
+                    'coursename' => format_string($course->shortname),
+                    'groupid' => $groupid,
+                    'userid' => $USER->id,
+                    'gradingcomponent' => $forumgradeitem->get_grading_component_name(),
+                    'gradingcomponentsubtype' => $forumgradeitem->get_grading_component_subtype(),
+                ];
+                echo $OUTPUT->render_from_template('mod_forum/grades/view_grade_button', $gradeobj);
+            }
+        }
         $discussion = $discussionvault->get_last_discussion_in_forum($forum);
         $discussioncount = $discussionvault->get_count_discussions_in_forum($forum);
         $hasmultiplediscussions = $discussioncount > 1;
@@ -160,16 +221,31 @@ switch ($forum->get_type()) {
                 $orderpostsby
             );
         echo $discussionsrenderer->render($USER, $post, $replies);
+
+        if (!$CFG->forum_usermarksread && forum_tp_is_tracked($forumrecord, $USER)) {
+            $postids = array_map(function($post) {
+                return $post->get_id();
+            }, array_merge([$post], array_values($replies)));
+            forum_tp_mark_posts_read($USER, $postids);
+        }
         break;
     case 'blog':
         $discussionsrenderer = $rendererfactory->get_blog_discussion_list_renderer($forum);
         // Blog forums always show discussions newest first.
         echo $discussionsrenderer->render($USER, $cm, $groupid, $discussionlistvault::SORTORDER_CREATED_DESC,
             $pageno, $pagesize);
+
+        if (!$CFG->forum_usermarksread && forum_tp_is_tracked($forumrecord, $USER)) {
+            $discussions = mod_forum_get_discussion_summaries($forum, $USER, $groupid, null, $pageno, $pagesize);
+            $firstpostids = array_map(function($discussion) {
+                return $discussion->get_first_post()->get_id();
+            }, array_values($discussions));
+            forum_tp_mark_posts_read($USER, $firstpostids);
+        }
         break;
     default:
         $discussionsrenderer = $rendererfactory->get_discussion_list_renderer($forum);
-        echo $discussionsrenderer->render($USER, $cm, $groupid, $sortorder, $pageno, $pagesize);
+        echo $discussionsrenderer->render($USER, $cm, $groupid, $sortorder, $pageno, $pagesize, $displaymode);
 }
 
 echo $OUTPUT->footer();
