@@ -48,6 +48,7 @@ class qformat_xml_test extends question_testcase {
         $q = new stdClass();
         $q->id = 0;
         $q->contextid = 0;
+        $q->idnumber = null;
         $q->category = 0;
         $q->parent = 0;
         $q->questiontextformat = FORMAT_HTML;
@@ -342,6 +343,7 @@ END;
         $qdata->length = 0;
         $qdata->penalty = 0;
         $qdata->hidden = 0;
+        $qdata->idnumber = null;
 
         $exporter = new qformat_xml();
         $xml = $exporter->writequestion($qdata);
@@ -360,6 +362,7 @@ END;
     <defaultgrade>0</defaultgrade>
     <penalty>0</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
   </question>
 ';
 
@@ -405,6 +408,7 @@ END;
         $expectedq->responsefieldlines = 15;
         $expectedq->attachments = 0;
         $expectedq->attachmentsrequired = 0;
+        $expectedq->filetypeslist = null;
         $expectedq->graderinfo['text'] = '';
         $expectedq->graderinfo['format'] = FORMAT_MOODLE;
         $expectedq->responsetemplate['text'] = '';
@@ -464,6 +468,7 @@ END;
         $expectedq->responsefieldlines = 42;
         $expectedq->attachments = -1;
         $expectedq->attachmentsrequired = 1;
+        $expectedq->filetypeslist = null;
         $expectedq->graderinfo['text'] = '<p>Grade <b>generously</b>!</p>';
         $expectedq->graderinfo['format'] = FORMAT_HTML;
         $expectedq->responsetemplate['text'] = '<p>Here is something <b>really</b> interesting.</p>';
@@ -487,6 +492,7 @@ END;
         $qdata->length = 1;
         $qdata->penalty = 0;
         $qdata->hidden = 0;
+        $qdata->idnumber = null;
         $qdata->options = new stdClass();
         $qdata->options->id = 456;
         $qdata->options->questionid = 123;
@@ -499,6 +505,7 @@ END;
         $qdata->options->graderinfoformat = FORMAT_HTML;
         $qdata->options->responsetemplate = '<p>Here is something <b>really</b> interesting.</p>';
         $qdata->options->responsetemplateformat = FORMAT_HTML;
+        $qdata->options->filetypeslist = '.pdf,.zip.,.docx';
         $exporter = new qformat_xml();
         $xml = $exporter->writequestion($qdata);
 
@@ -516,11 +523,13 @@ END;
     <defaultgrade>1</defaultgrade>
     <penalty>0</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
     <responseformat>monospaced</responseformat>
     <responserequired>0</responserequired>
     <responsefieldlines>42</responsefieldlines>
     <attachments>-1</attachments>
     <attachmentsrequired>1</attachmentsrequired>
+    <filetypeslist>.pdf,.zip.,.docx</filetypeslist>
     <graderinfo format="html">
       <text><![CDATA[<p>Grade <b>generously</b>!</p>]]></text>
     </graderinfo>
@@ -649,6 +658,7 @@ END;
         $qdata->length = 1;
         $qdata->penalty = 0.3333333;
         $qdata->hidden = 0;
+        $qdata->idnumber = null;
 
         $qdata->options = new stdClass();
         $qdata->options->shuffleanswers = 1;
@@ -709,6 +719,7 @@ END;
     <defaultgrade>1</defaultgrade>
     <penalty>0.3333333</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
     <shuffleanswers>true</shuffleanswers>
     <correctfeedback format="html">
       <text>Well done.</text>
@@ -880,11 +891,13 @@ END;
         $qdata->length = 1;
         $qdata->penalty = 0.3333333;
         $qdata->hidden = 0;
+        $qdata->idnumber = null;
 
         $qdata->options = new stdClass();
         $qdata->options->single = 0;
         $qdata->options->shuffleanswers = 0;
         $qdata->options->answernumbering = 'abc';
+        $qdata->options->showstandardinstruction = 0;
         $qdata->options->correctfeedback = '<p>Your answer is correct.</p>';
         $qdata->options->correctfeedbackformat = FORMAT_HTML;
         $qdata->options->partiallycorrectfeedback = '<p>Your answer is partially correct.</p>';
@@ -922,9 +935,11 @@ END;
     <defaultgrade>2</defaultgrade>
     <penalty>0.3333333</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
     <single>false</single>
     <shuffleanswers>false</shuffleanswers>
     <answernumbering>abc</answernumbering>
+    <showstandardinstruction>0</showstandardinstruction>
     <correctfeedback format="html">
       <text><![CDATA[<p>Your answer is correct.</p>]]></text>
     </correctfeedback>
@@ -1053,6 +1068,7 @@ END;
         $qdata->length = 1;
         $qdata->penalty = 0.1;
         $qdata->hidden = 0;
+        $qdata->idnumber = null;
 
         $qdata->options = new stdClass();
         $qdata->options->answers = array(
@@ -1083,6 +1099,7 @@ END;
     <defaultgrade>1</defaultgrade>
     <penalty>0.1</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
     <answer fraction="100" format="plain_text">
       <text>42</text>
       <feedback format="html">
@@ -1183,6 +1200,7 @@ END;
         $qdata->length = 1;
         $qdata->penalty = 0.3333333;
         $qdata->hidden = 0;
+        $qdata->idnumber = null;
 
         $qdata->options = new stdClass();
         $qdata->options->usecase = 0;
@@ -1214,6 +1232,7 @@ END;
     <defaultgrade>1</defaultgrade>
     <penalty>0.3333333</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
     <usecase>0</usecase>
     <answer fraction="100" format="plain_text">
       <text>Beta</text>
@@ -1290,6 +1309,59 @@ END;
         $this->assert(new question_check_specified_fields_expectation($expectedq), $q);
     }
 
+    public function test_import_truefalse_with_idnumber() {
+        $xml = '  <question type="truefalse">
+    <name>
+      <text>True false question</text>
+    </name>
+    <questiontext format="html">
+      <text>The answer is true.</text>
+    </questiontext>
+    <generalfeedback>
+      <text>General feedback: You should have chosen true.</text>
+    </generalfeedback>
+    <defaultgrade>1</defaultgrade>
+    <penalty>1</penalty>
+    <hidden>0</hidden>
+    <idnumber>TestIdNum1</idnumber>
+    <answer fraction="100">
+      <text>true</text>
+      <feedback>
+        <text>Well done!</text>
+      </feedback>
+    </answer>
+    <answer fraction="0">
+      <text>false</text>
+      <feedback>
+        <text>Doh!</text>
+      </feedback>
+    </answer>
+  </question>';
+        $xmldata = xmlize($xml);
+
+        $importer = new qformat_xml();
+        $q = $importer->import_truefalse($xmldata['question']);
+
+        $expectedq = new stdClass();
+        $expectedq->qtype = 'truefalse';
+        $expectedq->name = 'True false question';
+        $expectedq->questiontext = 'The answer is true.';
+        $expectedq->questiontextformat = FORMAT_HTML;
+        $expectedq->generalfeedback = 'General feedback: You should have chosen true.';
+        $expectedq->defaultmark = 1;
+        $expectedq->length = 1;
+        $expectedq->penalty = 1;
+        $expectedq->idnumber = 'TestIdNum1';
+
+        $expectedq->feedbacktrue = array('text' => 'Well done!',
+                'format' => FORMAT_HTML);
+        $expectedq->feedbackfalse = array('text' => 'Doh!',
+                'format' => FORMAT_HTML);
+        $expectedq->correctanswer = true;
+
+        $this->assert(new question_check_specified_fields_expectation($expectedq), $q);
+    }
+
     public function test_export_truefalse() {
         $qdata = new stdClass();
         $qdata->id = 12;
@@ -1304,6 +1376,7 @@ END;
         $qdata->length = 1;
         $qdata->penalty = 1;
         $qdata->hidden = 0;
+        $qdata->idnumber = null;
 
         $qdata->options = new stdClass();
         $qdata->options->answers = array(
@@ -1330,6 +1403,67 @@ END;
     <defaultgrade>1</defaultgrade>
     <penalty>1</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
+    <answer fraction="100" format="plain_text">
+      <text>true</text>
+      <feedback format="html">
+        <text>Well done!</text>
+      </feedback>
+    </answer>
+    <answer fraction="0" format="plain_text">
+      <text>false</text>
+      <feedback format="html">
+        <text>Doh!</text>
+      </feedback>
+    </answer>
+  </question>
+';
+
+        $this->assert_same_xml($expectedxml, $xml);
+    }
+
+    public function test_export_truefalse_with_idnumber() {
+        $qdata = new stdClass();
+        $qdata->id = 12;
+        $qdata->contextid = \context_system::instance()->id;
+        $qdata->qtype = 'truefalse';
+        $qdata->name = 'True false question';
+        $qdata->questiontext = 'The answer is true.';
+        $qdata->questiontextformat = FORMAT_HTML;
+        $qdata->generalfeedback = 'General feedback: You should have chosen true.';
+        $qdata->generalfeedbackformat = FORMAT_HTML;
+        $qdata->defaultmark = 1;
+        $qdata->length = 1;
+        $qdata->penalty = 1;
+        $qdata->hidden = 0;
+        $qdata->idnumber = 'TestIDNum2';
+
+        $qdata->options = new stdClass();
+        $qdata->options->answers = array(
+                1 => new question_answer(1, 'True', 1, 'Well done!', FORMAT_HTML),
+                2 => new question_answer(2, 'False', 0, 'Doh!', FORMAT_HTML),
+        );
+        $qdata->options->trueanswer = 1;
+        $qdata->options->falseanswer = 2;
+
+        $exporter = new qformat_xml();
+        $xml = $exporter->writequestion($qdata);
+
+        $expectedxml = '<!-- question: 12  -->
+  <question type="truefalse">
+    <name>
+      <text>True false question</text>
+    </name>
+    <questiontext format="html">
+      <text>The answer is true.</text>
+    </questiontext>
+    <generalfeedback format="html">
+      <text>General feedback: You should have chosen true.</text>
+    </generalfeedback>
+    <defaultgrade>1</defaultgrade>
+    <penalty>1</penalty>
+    <hidden>0</hidden>
+    <idnumber>TestIDNum2</idnumber>
     <answer fraction="100" format="plain_text">
       <text>true</text>
       <feedback format="html">
@@ -1474,6 +1608,7 @@ END;
     </generalfeedback>
     <penalty>0.3333333</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
     <hint format="html">
       <text>Hint 1</text>
     </hint>
@@ -1505,6 +1640,7 @@ END;
     </generalfeedback>
     <penalty>0.3333333</penalty>
     <hidden>0</hidden>
+    <idnumber></idnumber>
   </question>
 ';
 
@@ -1583,4 +1719,41 @@ END;
         $this->assertEquals('/myfolder/', $file->filepath);
         $this->assertEquals(6,            $file->size);
     }
+
+    public function test_create_dummy_question() {
+
+        $testobject = new mock_qformat_xml();
+        $categoryname = 'name1';
+        $categoryinfo = new stdClass();
+        $categoryinfo->info = 'info1';
+        $categoryinfo->infoformat = 'infoformat1';
+        $categoryinfo->idnumber = null;
+        $dummyquestion = $testobject->mock_create_dummy_question_representing_category($categoryname, $categoryinfo);
+
+        $this->assertEquals('category', $dummyquestion->qtype);
+        $this->assertEquals($categoryname, $dummyquestion->category);
+        $this->assertEquals($categoryinfo->info, $dummyquestion->info);
+        $this->assertEquals($categoryinfo->infoformat, $dummyquestion->infoformat);
+        $this->assertEquals('Switch category to ' . $categoryname, $dummyquestion->name);
+        $this->assertEquals(0, $dummyquestion->id);
+        $this->assertEquals('', $dummyquestion->questiontextformat);
+        $this->assertEquals(0, $dummyquestion->contextid);
+    }
+}
+
+/**
+ * Class mock_qformat_xml exists only to enable testing of the create dummy question category.
+ * @package    qformat_xml
+ * @copyright  2018 The Open University
+ * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ */
+class mock_qformat_xml extends qformat_xml {
+    /**
+     * Make public an otherwise protected function.
+     * @param string $categoryname the name of the category
+     * @param object $categoryinfo description of the category
+     */
+    public function mock_create_dummy_question_representing_category(string $categoryname, $categoryinfo) {
+        return $this->create_dummy_question_representing_category($categoryname, $categoryinfo);
+    }
 }
