@@ -30,9 +30,9 @@
  */
 
 
-require_once(dirname(__FILE__) . '/../config.php');
+require_once(__DIR__ . '/../config.php');
 require_once($CFG->libdir . '/questionlib.php');
-require_once(dirname(__FILE__) . '/previewlib.php');
+require_once(__DIR__ . '/previewlib.php');
 
 /**
  * The maximum number of variants previewable. If there are more variants than this for a question
@@ -95,7 +95,7 @@ if ($previewid) {
     }
 
     $slot = $quba->get_first_question_number();
-    $usedquestion = $quba->get_question($slot);
+    $usedquestion = $quba->get_question($slot, false);
     if ($usedquestion->id != $question->id) {
         print_error('questionidmismatch', 'question');
     }
@@ -255,25 +255,34 @@ echo $quba->render_question($slot, $options, $displaynumber);
 // Finish the question form.
 echo html_writer::start_tag('div', array('id' => 'previewcontrols', 'class' => 'controls'));
 echo html_writer::empty_tag('input', $restartdisabled + array('type' => 'submit',
-        'name' => 'restart', 'value' => get_string('restart', 'question')));
+        'name' => 'restart', 'value' => get_string('restart', 'question'), 'class' => 'btn btn-secondary'));
 echo html_writer::empty_tag('input', $finishdisabled  + array('type' => 'submit',
-        'name' => 'save',    'value' => get_string('save', 'question')));
+        'name' => 'save',    'value' => get_string('save', 'question'), 'class' => 'btn btn-secondary'));
 echo html_writer::empty_tag('input', $filldisabled    + array('type' => 'submit',
-        'name' => 'fill',    'value' => get_string('fillincorrect', 'question')));
+        'name' => 'fill',    'value' => get_string('fillincorrect', 'question'), 'class' => 'btn btn-secondary'));
 echo html_writer::empty_tag('input', $finishdisabled  + array('type' => 'submit',
-        'name' => 'finish',  'value' => get_string('submitandfinish', 'question')));
+        'name' => 'finish',  'value' => get_string('submitandfinish', 'question'), 'class' => 'btn btn-secondary'));
 echo html_writer::end_tag('div');
 echo html_writer::end_tag('form');
 
 // Output the technical info.
-print_collapsible_region_start('', 'techinfo', get_string('technicalinfo', 'question') .
-        $OUTPUT->help_icon('technicalinfo', 'question'),
-        'core_question_preview_techinfo_collapsed', true);
+print_collapsible_region_start('', 'techinfo', get_string('technicalinfo', 'question'),
+        'core_question_preview_techinfo_collapsed', true, false, $OUTPUT->help_icon('technicalinfo', 'question'));
 foreach ($technical as $info) {
     echo html_writer::tag('p', $info, array('class' => 'notifytiny'));
 }
 print_collapsible_region_end();
 
+// Output a link to export this single question.
+if (question_has_capability_on($question, 'view')) {
+    echo html_writer::link(question_get_export_single_question_url($question),
+            get_string('exportonequestion', 'question'));
+}
+
+// Log the preview of this question.
+$event = \core\event\question_viewed::create_from_question_instance($question, $context);
+$event->trigger();
+
 // Display the settings form.
 $optionsform->display();
 
